@@ -465,15 +465,6 @@ chat_message_received (EmpathyChat *chat, EmpathyMessage *message)
 		empathy_contact_get_name (sender),
 		empathy_contact_get_handle (sender));
 
-	if (priv->id) {
-		gboolean is_chatroom;
-
-		is_chatroom = priv->handle_type == TP_HANDLE_TYPE_ROOM;
-		empathy_log_manager_add_message (priv->log_manager,
-						 priv->id, is_chatroom,
-						 message);
-	}
-
 	empathy_chat_view_append_message (chat->view, message);
 
 	/* We received a message so the contact is no longer composing */
@@ -1035,14 +1026,31 @@ chat_input_populate_popup_cb (GtkTextView *view,
 	}
 }
 
+static gboolean
+chat_log_filter (EmpathyMessage *message,
+		 gpointer user_data)
+{
+	EmpathyChat *chat = (EmpathyChat *) user_data;
+	EmpathyChatPriv *priv = GET_PRIV (chat);
+	const GList *pending;
+
+	pending = empathy_tp_chat_get_pending_messages (priv->tp_chat);
+
+	for (; pending; pending = g_list_next (pending)) {
+		if (empathy_message_equal (message, pending->data)) {
+			return FALSE;
+		}
+	}
+
+	return TRUE;
+}
+
 static void
 chat_add_logs (EmpathyChat *chat)
 {
 	EmpathyChatPriv *priv = GET_PRIV (chat);
 	gboolean         is_chatroom;
 	GList           *messages, *l;
-	guint            num_messages;
-	guint            i;
 
 	if (!priv->id) {
 		return;
@@ -1053,25 +1061,20 @@ chat_add_logs (EmpathyChat *chat)
 
 	/* Add messages from last conversation */
 	is_chatroom = priv->handle_type == TP_HANDLE_TYPE_ROOM;
-	messages = empathy_log_manager_get_last_messages (priv->log_manager,
-							  priv->account,
-							  priv->id,
-							  is_chatroom);
-	num_messages  = g_list_length (messages);
 
-	/* Only keep the 10 last messages */
-	for (i = 0; num_messages - i > 10; i++) {
-		EmpathyMessage *message;
+	messages = empathy_log_manager_get_filtered_messages (priv->log_manager,
+							      priv->account,
+							      priv->id,
+							      is_chatroom,
+							      5,
+							      chat_log_filter,
+							      chat);
 
-		message = messages->data;
-		messages = g_list_remove (messages, message);
-		g_object_unref (message);
-	}
-
-	for (l = messages; l; l = l->next) {
+	for (l = messages; l; l = g_list_next (l)) {
 		empathy_chat_view_append_message (chat->view, l->data);
 		g_object_unref (l->data);
 	}
+
 	g_list_free (messages);
 
 	/* Turn back on scrolling */
