@@ -26,6 +26,7 @@
 #include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
 
+#include <libempathy/empathy-call-factory.h>
 #include <libempathy/empathy-log-manager.h>
 #include <libempathy/empathy-dispatcher.h>
 #include <libempathy/empathy-utils.h>
@@ -110,6 +111,14 @@ empathy_contact_menu_new (EmpathyContact             *contact,
 	return menu;
 }
 
+static void
+empathy_contact_chat_menu_item_activated (GtkMenuItem *item,
+	EmpathyContact *contact)
+{
+  empathy_dispatcher_chat_with_contact (contact, NULL, NULL);
+}
+
+
 GtkWidget *
 empathy_contact_chat_menu_item_new (EmpathyContact *contact)
 {
@@ -124,13 +133,23 @@ empathy_contact_chat_menu_item_new (EmpathyContact *contact)
 	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
 	gtk_widget_show (image);
 
-	g_signal_connect_swapped (item, "activate",
-				  G_CALLBACK (empathy_dispatcher_chat_with_contact),
+	g_signal_connect (item, "activate",
+				  G_CALLBACK (empathy_contact_chat_menu_item_activated),
 				  contact);
 	
 	return item;
 }
 
+static void
+empathy_contact_call_menu_item_activated (GtkMenuItem *item,
+	EmpathyContact *contact)
+{
+	EmpathyCallFactory *factory;
+
+	factory = empathy_call_factory_get ();
+	empathy_call_factory_new_call (factory, contact);
+}
+
 GtkWidget *
 empathy_contact_call_menu_item_new (EmpathyContact *contact)
 {
@@ -146,8 +165,8 @@ empathy_contact_call_menu_item_new (EmpathyContact *contact)
 	gtk_widget_set_sensitive (item, empathy_contact_can_voip (contact));
 	gtk_widget_show (image);
 
-	g_signal_connect_swapped (item, "activate",
-				  G_CALLBACK (empathy_dispatcher_call_with_contact),
+	g_signal_connect (item, "activate",
+				  G_CALLBACK (empathy_contact_call_menu_item_activated),
 				  contact);
 	
 	return item;
@@ -171,7 +190,7 @@ empathy_contact_log_menu_item_new (EmpathyContact *contact)
 
 	g_return_val_if_fail (EMPATHY_IS_CONTACT (contact), NULL);
 
-	manager = empathy_log_manager_new ();
+	manager = empathy_log_manager_dup_singleton ();
 	have_log = empathy_log_manager_exists (manager,
 					       empathy_contact_get_account (contact),
 					       empathy_contact_get_id (contact),
@@ -301,20 +320,20 @@ room_sub_menu_activate_cb (GtkWidget *item,
 {
 	TpHandle handle;
 	GArray handles = {(gchar *) &handle, 1};
+	EmpathyTpChat *chat;
 	TpChannel *channel;
 
-	g_object_get (data->chatroom, "tp-channel", &channel, NULL);
-	if (channel == NULL) {
+	chat = empathy_chatroom_get_tp_chat (data->chatroom);
+	if (chat == NULL) {
 		/* channel was invalidated. Ignoring */
 		return;
 	}
 
 	/* send invitation */
 	handle = empathy_contact_get_handle (data->contact);
+	channel = empathy_tp_chat_get_channel (chat);
 	tp_cli_channel_interface_group_call_add_members (channel, -1, &handles,
 		_("Inviting to this room"), NULL, NULL, NULL, NULL);
-
-	g_object_unref (channel);
 }
 
 static GtkWidget *
@@ -352,7 +371,7 @@ empathy_contact_invite_menu_item_new (EmpathyContact *contact)
 					      GTK_ICON_SIZE_MENU);
 	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
 
-	mgr = empathy_chatroom_manager_new (NULL);
+	mgr = empathy_chatroom_manager_dup_singleton (NULL);
 	rooms = empathy_chatroom_manager_get_chatrooms (mgr,
 		empathy_contact_get_account (contact));
 
@@ -362,17 +381,13 @@ empathy_contact_invite_menu_item_new (EmpathyContact *contact)
 
 	for (l = rooms; l != NULL; l = g_list_next (l)) {
 		EmpathyChatroom *chatroom = l->data;
-		TpChannel *channel;
 
-		g_object_get (chatroom, "tp-channel", &channel, NULL);
-		if (channel != NULL) {
+		if (empathy_chatroom_get_tp_chat (chatroom) != NULL) {
 			have_rooms = TRUE;
 
 			room_item = create_room_sub_menu (contact, chatroom);
 			gtk_menu_shell_append (submenu_shell, room_item);
 			gtk_widget_show (room_item);
-
-			g_object_unref (channel);
 		}
 	}
 
