@@ -339,6 +339,8 @@ empathy_video_widget_element_added_cb (FsElementAddedNotifier *notifier,
   if (priv->overlay == NULL && GST_IS_X_OVERLAY (element))
     {
       priv->overlay = element;
+      g_object_add_weak_pointer (G_OBJECT (element),
+        (gpointer) &priv->overlay);
       empathy_video_widget_element_set_sink_properties_unlocked (self);
       gst_x_overlay_expose (GST_X_OVERLAY (priv->overlay));
     }
@@ -362,6 +364,7 @@ empathy_video_widget_sync_message_cb (GstBus *bus, GstMessage *message,
 
   if (gst_structure_has_name (s, "prepare-xwindow-id"))
     {
+      g_assert (GTK_WIDGET_REALIZED (GTK_WIDGET (self)));
       gst_x_overlay_set_xwindow_id (GST_X_OVERLAY (priv->overlay),
         GDK_WINDOW_XID (GTK_WIDGET (self)->window));
     }
@@ -377,7 +380,11 @@ empathy_video_widget_expose_event (GtkWidget *widget, GdkEventExpose *event)
     return TRUE;
 
   if (priv->overlay == NULL)
-    return TRUE;
+    {
+      gdk_window_clear_area (widget->window, 0, 0,
+        widget->allocation.width, widget->allocation.height);
+      return TRUE;
+    }
 
   gst_x_overlay_set_xwindow_id (GST_X_OVERLAY (priv->overlay),
     GDK_WINDOW_XID (widget->window));
