@@ -26,6 +26,8 @@
 #include <telepathy-glib/enums.h>
 
 #include "empathy-contact-manager.h"
+#include "empathy-account-manager.h"
+#include "empathy-contact-monitor.h"
 #include "empathy-contact-list.h"
 #include "empathy-utils.h"
 
@@ -35,8 +37,8 @@
 #define GET_PRIV(obj) EMPATHY_GET_PRIV (obj, EmpathyContactManager)
 typedef struct {
 	GHashTable     *lists;
-	MissionControl *mc;
-	gpointer        token;
+	EmpathyAccountManager *account_manager;
+	EmpathyContactMonitor *contact_monitor;
 } EmpathyContactManagerPriv;
 
 static void contact_manager_iface_init         (EmpathyContactListIface    *iface);
@@ -45,6 +47,8 @@ G_DEFINE_TYPE_WITH_CODE (EmpathyContactManager, empathy_contact_manager, G_TYPE_
 			 G_IMPLEMENT_INTERFACE (EMPATHY_TYPE_CONTACT_LIST,
 						contact_manager_iface_init));
 
+static EmpathyContactManager *manager_singleton = NULL;
+
 static void
 contact_manager_members_changed_cb (EmpathyTpContactList  *list,
 				    EmpathyContact        *contact,
@@ -159,23 +163,19 @@ contact_manager_add_account (EmpathyContactManager *manager,
 }
 
 static void
-contact_manager_status_changed_cb (MissionControl           *mc,
-				   TpConnectionStatus        status,
-				   McPresence                presence,
-				   TpConnectionStatusReason  reason,
-				   const gchar              *unique_name,
-				   EmpathyContactManager    *manager)
+contact_manager_connection_changed_cb (EmpathyAccountManager *account_manager,
+				       McAccount *account,
+				       TpConnectionStatusReason  reason,
+				       TpConnectionStatus current,
+				       TpConnectionStatus previous,
+				       EmpathyContactManager *manager)
 {
-	McAccount *account;
-
-	if (status != TP_CONNECTION_STATUS_CONNECTED) {
+	if (current != TP_CONNECTION_STATUS_CONNECTED) {
 		/* We only care about newly connected accounts */
 		return;
 	}
 
-	account = mc_account_lookup (unique_name);
 	contact_manager_add_account (manager, account);
-	g_object_unref (account);
 }
 
 static void
@@ -183,12 +183,39 @@ contact_manager_finalize (GObject *object)
 {
 	EmpathyContactManagerPriv *priv = GET_PRIV (object);
 
-	empathy_disconnect_account_status_changed (priv->token);
 	g_hash_table_foreach (priv->lists,
 			      contact_manager_disconnect_foreach,
 			      object);
 	g_hash_table_destroy (priv->lists);
-	g_object_unref (priv->mc);
+
+	g_signal_handlers_disconnect_by_func (priv->account_manager,
+					      contact_manager_connection_changed_cb,
+					      object);
+	g_object_unref (priv->account_manager);
+
+	if (priv->contact_monitor) {
+		g_object_unref (priv->contact_monitor);
+	}
+}
+
+static GObject *
+contact_manager_constructor (GType type,
+			     guint n_props,
+			     GObjectConstructParam *props)
+{
+	GObject *retval;
+
+	if (manager_singleton) {
+		retval = g_object_ref (manager_singleton);
+	} else {
+		retval = G_OBJECT_CLASS (empathy_contact_manager_parent_class)->constructor
+			(type, n_props, props);
+
+		manager_singleton = EMPATHY_CONTACT_MANAGER (retval);
+		g_object_add_weak_pointer (retval, (gpointer) &manager_singleton);
+	}
+
+	return retval;
 }
 
 static void
@@ -197,6 +224,7 @@ empathy_contact_manager_class_init (EmpathyContactManagerClass *klass)
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
 	object_class->finalize = contact_manager_finalize;
+	object_class->constructor = contact_manager_constructor;
 
 	g_type_class_add_private (object_class, sizeof (EmpathyContactManagerPriv));
 }
@@ -205,6 +233,7 @@ static void
 empathy_contact_manager_init (EmpathyContactManager *manager)
 {
 	GSList                    *accounts, *l;
+	MissionControl            *mc;
 	EmpathyContactManagerPriv *priv = G_TYPE_INSTANCE_GET_PRIVATE (manager,
 		EMPATHY_TYPE_CONTACT_MANAGER, EmpathyContactManagerPriv);
 
@@ -213,34 +242,30 @@ empathy_contact_manager_init (EmpathyContactManager *manager)
 					     empathy_account_equal,
 					     (GDestroyNotify) g_object_unref,
 					     (GDestroyNotify) g_object_unref);
+	priv->account_manager = empathy_account_manager_dup_singleton ();
+	priv->contact_monitor = NULL;
 
-	priv->mc = empathy_mission_control_new ();
-	priv->token = empathy_connect_to_account_status_changed (priv->mc,
-		G_CALLBACK (contact_manager_status_changed_cb),
-		manager, NULL);
+	g_signal_connect (priv->account_manager,
+			  "account-connection-changed",
+			  G_CALLBACK (contact_manager_connection_changed_cb), manager);
+
+	mc = empathy_mission_control_dup_singleton ();
 
 	/* Get ContactList for existing connections */
-	accounts = mission_control_get_online_connections (priv->mc, NULL);
+	accounts = mission_control_get_online_connections (mc, NULL);
 	for (l = accounts; l; l = l->next) {
 		contact_manager_add_account (manager, l->data);
 		g_object_unref (l->data);
 	}
+
 	g_slist_free (accounts);
+	g_object_unref (mc);
 }
 
 EmpathyContactManager *
-empathy_contact_manager_new (void)
+empathy_contact_manager_dup_singleton (void)
 {
-	static EmpathyContactManager *manager = NULL;
-
-	if (!manager) {
-		manager = g_object_new (EMPATHY_TYPE_CONTACT_MANAGER, NULL);
-		g_object_add_weak_pointer (G_OBJECT (manager), (gpointer) &manager);
-	} else {
-		g_object_ref (manager);
-	}
-
-	return manager;
+	return g_object_new (EMPATHY_TYPE_CONTACT_MANAGER, NULL);
 }
 
 EmpathyTpContactList *
@@ -319,6 +344,18 @@ contact_manager_get_members (EmpathyContactList *manager)
 	return contacts;
 }
 
+static EmpathyContactMonitor *
+contact_manager_get_monitor (EmpathyContactList *manager)
+{
+	EmpathyContactManagerPriv *priv = GET_PRIV (manager);
+
+	if (priv->contact_monitor == NULL) {
+		priv->contact_monitor = empathy_contact_monitor_new_for_iface (manager);
+	}
+
+	return priv->contact_monitor;
+}
+
 static void
 contact_manager_get_pendings_foreach (McAccount             *account,
 				      EmpathyTpContactList  *list,
@@ -498,6 +535,7 @@ contact_manager_iface_init (EmpathyContactListIface *iface)
 	iface->add               = contact_manager_add;
 	iface->remove            = contact_manager_remove;
 	iface->get_members       = contact_manager_get_members;
+	iface->get_monitor       = contact_manager_get_monitor;
 	iface->get_pendings      = contact_manager_get_pendings;
 	iface->get_all_groups    = contact_manager_get_all_groups;
 	iface->get_groups        = contact_manager_get_groups;
