@@ -31,6 +31,7 @@
 #include <libempathy/empathy-contact.h>
 #include <libempathy/empathy-utils.h>
 #include <libempathy/empathy-account-manager.h>
+#include <libempathy/empathy-dispatcher.h>
 #include <libempathy/empathy-chatroom-manager.h>
 #include <libempathy/empathy-chatroom.h>
 #include <libempathy/empathy-contact-list.h>
@@ -49,6 +50,8 @@
 #include <libempathy-gtk/empathy-new-message-dialog.h>
 #include <libempathy-gtk/empathy-gtk-enum-types.h>
 
+#include <libmissioncontrol/mission-control.h>
+
 #include "empathy-accounts-dialog.h"
 #include "empathy-main-window.h"
 #include "ephy-spinner.h"
@@ -96,6 +99,7 @@ typedef struct {
 	GtkWidget              *room_join_favorites;
 	GtkWidget              *edit_context;
 	GtkWidget              *edit_context_separator;
+	GtkWidget              *chat_history_menu_item;
 
 	guint                   size_timeout_id;
 	GHashTable             *errors;
@@ -104,6 +108,8 @@ typedef struct {
 	GList                  *widgets_connected;
 } EmpathyMainWindow;
 
+static EmpathyMainWindow *window = NULL;
+
 static void     main_window_destroy_cb                         (GtkWidget                *widget,
 								EmpathyMainWindow        *window);
 static void     main_window_favorite_chatroom_menu_setup       (EmpathyMainWindow        *window);
@@ -176,6 +182,9 @@ static void     main_window_notify_compact_contact_list_cb     (EmpathyConf
 static void     main_window_notify_sort_criterium_cb           (EmpathyConf              *conf,
 								const gchar              *key,
 								EmpathyMainWindow        *window);
+static void     main_window_account_created_or_deleted_cb      (EmpathyAccountManager    *manager,
+								McAccount                *account,
+								EmpathyMainWindow        *window);
 
 static void
 main_window_flash_stop (EmpathyMainWindow *window)
@@ -283,9 +292,6 @@ main_window_flash_cb (EmpathyMainWindow *window)
 static void
 main_window_flash_start (EmpathyMainWindow *window)
 {
-	empathy_sound_play (GTK_WIDGET (window->window),
-			    EMPATHY_SOUND_CONVERSATION_NEW);
-
 	if (window->flash_timeout_id != 0) {
 		return;
 	}
@@ -479,10 +485,15 @@ main_window_contact_presence_changed_cb (EmpathyContactMonitor *monitor,
 	}
 }
 
+GtkWidget *
+empathy_main_window_get (void)
+{
+  return window != NULL ? window->window : NULL;
+}
+
 GtkWidget *
 empathy_main_window_show (void)
 {
-	static EmpathyMainWindow *window = NULL;
 	EmpathyContactList       *list_iface;
 	EmpathyContactMonitor    *monitor;
 	GladeXML                 *glade;
@@ -521,6 +532,7 @@ empathy_main_window_show (void)
 				       "edit_context_separator", &window->edit_context_separator,
 				       "presence_toolbar", &window->presence_toolbar,
 				       "roster_scrolledwindow", &sw,
+				       "chat_history", &window->chat_history_menu_item,
 				       NULL);
 	g_free (filename);
 
@@ -548,7 +560,7 @@ empathy_main_window_show (void)
 	main_window_connection_items_setup (window, glade);
 	g_object_unref (glade);
 
-	window->mc = empathy_mission_control_new ();
+	window->mc = empathy_mission_control_dup_singleton ();
 	window->account_manager = empathy_account_manager_dup_singleton ();
 
 	g_signal_connect (window->account_manager,
@@ -599,7 +611,7 @@ empathy_main_window_show (void)
 	/* Set up contact list. */
 	empathy_status_presets_get_all ();
 
-	list_iface = EMPATHY_CONTACT_LIST (empathy_contact_manager_new ());
+	list_iface = EMPATHY_CONTACT_LIST (empathy_contact_manager_dup_singleton ());
 	monitor = empathy_contact_list_get_monitor (list_iface);
 	window->list_store = empathy_contact_list_store_new (list_iface);
 	window->list_view = empathy_contact_list_view_new (window->list_store,
@@ -639,7 +651,7 @@ empathy_main_window_show (void)
 	}
 
 	/* Enable event handling */
-	window->event_manager = empathy_event_manager_new ();
+	window->event_manager = empathy_event_manager_dup_singleton ();
 	g_signal_connect (window->event_manager, "event-added",
 			  G_CALLBACK (main_window_event_added_cb),
 			  window);
@@ -647,6 +659,14 @@ empathy_main_window_show (void)
 			  G_CALLBACK (main_window_event_removed_cb),
 			  window);
 
+	g_signal_connect (window->account_manager, "account-created",
+			  G_CALLBACK (main_window_account_created_or_deleted_cb),
+			  window);
+	g_signal_connect (window->account_manager, "account-deleted",
+			  G_CALLBACK (main_window_account_created_or_deleted_cb),
+			  window);
+	main_window_account_created_or_deleted_cb (window->account_manager, NULL, window);
+
 	l = empathy_event_manager_get_events (window->event_manager);
 	while (l) {
 		main_window_event_added_cb (window->event_manager,
@@ -740,7 +760,7 @@ main_window_favorite_chatroom_menu_setup (EmpathyMainWindow *window)
 {
 	GList *chatrooms, *l;
 
-	window->chatroom_manager = empathy_chatroom_manager_new (NULL);
+	window->chatroom_manager = empathy_chatroom_manager_dup_singleton (NULL);
 	chatrooms = empathy_chatroom_manager_get_chatrooms (window->chatroom_manager, NULL);
 	window->room_menu = gtk_menu_item_get_submenu (GTK_MENU_ITEM (window->room));
 
@@ -843,25 +863,20 @@ main_window_favorite_chatroom_join (EmpathyChatroom *chatroom)
 {
 	MissionControl *mc;
 	McAccount      *account;
-	TpConnection   *connection;
 	const gchar    *room;
 
-	mc = empathy_mission_control_new ();
+	mc = empathy_mission_control_dup_singleton ();
 	account = empathy_chatroom_get_account (chatroom);
 	room = empathy_chatroom_get_room (chatroom);
 
-	DEBUG ("Requesting channel for '%s'", room);
-
-	connection = mission_control_get_tpconnection (mc, account, NULL);
-	if (connection) {
-		tp_connection_run_until_ready (connection, TRUE, NULL, NULL);	
-		empathy_connection_request_channel (connection, -1,
-						    TP_IFACE_CHANNEL_TYPE_TEXT,
-						    TP_HANDLE_TYPE_ROOM,
-						    room, TRUE,
-						    NULL, NULL, NULL, NULL);
-		g_object_unref (connection);
+	 if (mission_control_get_connection_status (mc, account, NULL) !=
+			TP_CONNECTION_STATUS_CONNECTED) {
+		return;
 	}
+
+	DEBUG ("Requesting channel for '%s'", room);
+	empathy_dispatcher_join_muc (account, room, NULL, NULL);
+
 	g_object_unref (mc);
 }
 
@@ -1009,7 +1024,7 @@ main_window_edit_personal_information_cb (GtkWidget         *widget,
 		McAccount             *account;
 
 		account = accounts->data;
-		factory = empathy_contact_factory_new ();
+		factory = empathy_contact_factory_dup_singleton ();
 		contact = empathy_contact_factory_get_user (factory, account);
 		empathy_contact_run_until_ready (contact,
 						 EMPATHY_CONTACT_READY_HANDLE |
@@ -1395,3 +1410,11 @@ main_window_notify_sort_criterium_cb (EmpathyConf       *conf,
 	}
 }
 
+static void
+main_window_account_created_or_deleted_cb (EmpathyAccountManager  *manager,
+					   McAccount              *account,
+					   EmpathyMainWindow      *window)
+{
+	gtk_widget_set_sensitive (GTK_WIDGET (window->chat_history_menu_item),
+		empathy_account_manager_get_count (manager) > 0);
+}
